/*
 *--------------------------------------
 * Program Name: DIAL
 * Author: LogicalJoe
 * Copyright: Copyright (C) 2017 LogicalJoe
 * License: LJL
 * Description: Impossible dial for the TI-84 Plus CE
 *--------------------------------------
 */

/* Keep these headers */
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <tice.h>

/* Standard headers (recommended) */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <LJ.h>

#include <graphx.h>
#include <keypadc.h>
#include <fileioc.h>

static const char *Dial_AppVar = "DialHigh";

bool key, prevkey;

/* Put your function prototypes here */
void inital_render(void);
void render_dial(void);
void update(void);
void die(void);
void game(void);
void death(void);
void passed(void);
void high_Disp(void);
void menu_draw(void);
void main_gamecode(void);

float a, b, c, go;
int length = 98, way, color, colorprev, q;
bool quit, line = false;
int highscore, score = 0;
typedef struct appVar1 {
    int Highscore;
} appVar1_t;
appVar1_t appVar;
ti_var_t file;
int x0, y0, x1, y1, x2, y2, x3, y3;

/* Put all your globals here */

void main(void) {
    /* Fill in the body of the main function here */
    
    ti_CloseAll();
    file = ti_Open(Dial_AppVar, "r");
    if (file) {
        ti_Read(&highscore, sizeof(highscore), 1, file);
    }
    else {
        highscore = 0;
    }
    
    /* Initialize the 8bpp graphics */
    gfx_Begin(gfx_8bpp);
    
    /* Render circle */
    inital_render();
    
    /* place the circle on the buffer */
    gfx_Blit(gfx_screen);
    
    gfx_SetTextTransparentColor(1);
    
    menu_draw();
    while (quit == false) {
        
        while (kb_AnyKey());
        while (!kb_AnyKey())
            kb_Scan();
        
        if (kb_Data[6] == kb_Clear)
            quit = true;
        if (kb_Data[1] & kb_Zoom)
            line = true;
        else
            line = false;
        if (quit == false) {
            game();
            menu_draw();
        }
    }
    
    /* archive the save file */
    file = ti_Open(Dial_AppVar, "r");
    if (file)
        ti_SetArchiveStatus(true, file);
    ti_CloseAll();
    
    /* Close the graphics */
    gfx_End();
}

/* Put other functions here */

void update(void) {
    
    b = a;
    
    if (way == 1){
        a += go;
        if (a > 360)
            a -= 360;
    }
    if (way == -1){
        a -= go;
        if (a < 0)
            a += 360;
    }
}

void render_dial(void) {
    gfx_SetColor(gfx_white);
    gfx_FillCircle(160, 120, 105);
    
    if (color == 1)
        gfx_SetColor(lj_red);
    if (color == 2)
        gfx_SetColor(lj_blue);
    if (color == 3)
        gfx_SetColor(lj_yellow);
    if (color == 4)
        gfx_SetColor(lj_green);
    /* length L a = (length*cos(a),length*sin(a),) */
    
    if (line == true)
        // thin one-line thick line:
        gfx_Line_NoClip(160, 120, length * cos(a * M_PI / 180.0) + 160, length * sin(a * M_PI / 180.0) + 120);
    

    
    
    if (line == false) {
        
        // end circles:
        gfx_FillCircle_NoClip(160, 120, 5);
        gfx_FillCircle_NoClip(length * cos(a * M_PI / 180.0) + 160, length * sin(a * M_PI / 180.0) + 120, 5);
        
        // two one-line thick side lines:
        gfx_Line_NoClip(5 * cos((a + 90) * M_PI / 180.0) + 160, 5 * sin((a + 90) * M_PI / 180.0) + 120, 5 * cos((a + 90) * M_PI / 180.0) + length * cos(a * M_PI / 180.0) + 160, 5 * sin((a + 90) * M_PI / 180.0) + length * sin(a * M_PI / 180.0) + 120);
        gfx_Line_NoClip(5 * cos((a - 90) * M_PI / 180.0) + 160, 5 * sin((a - 90) * M_PI / 180.0) + 120, 5 * cos((a - 90) * M_PI / 180.0) + length * cos(a * M_PI / 180.0) + 160, 5 * sin((a - 90) * M_PI / 180.0) + length * sin(a * M_PI / 180.0) + 120);
        
        // flood fill:
        if (color == 1)
            gfx_FloodFill(10 * cos(a * M_PI / 180.0) + 160, 10 * sin(a * M_PI / 180.0) + 120, lj_red);
        if (color == 2)
            gfx_FloodFill(10 * cos(a * M_PI / 180.0) + 160, 10 * sin(a * M_PI / 180.0) + 120, lj_blue);
        if (color == 3)
            gfx_FloodFill(10 * cos(a * M_PI / 180.0) + 160, 10 * sin(a * M_PI / 180.0) + 120, lj_yellow);
        if (color == 4)
            gfx_FloodFill(10 * cos(a * M_PI / 180.0) + 160, 10 * sin(a * M_PI / 180.0) + 120, lj_green);
        
    }
    high_Disp();
}

void inital_render(void) {
    gfx_SetDrawBuffer();
    gfx_SetColor(lj_red);
    gfx_FillCircle(160, 120, 119);
    gfx_SetColor(lj_white);
    gfx_FillCircle(160, 120, 105);
    gfx_BlitRectangle(gfx_buffer, 0, 0, 160, 120);
    
    gfx_SetColor(lj_blue);
    gfx_FillCircle(160, 120, 119);
    gfx_SetColor(lj_white);
    gfx_FillCircle(160, 120, 105);
    gfx_BlitRectangle(gfx_buffer, 160, 0, 160, 120);
    
    gfx_SetColor(lj_yellow);
    gfx_FillCircle(160, 120, 119);
    gfx_SetColor(lj_white);
    gfx_FillCircle(160, 120, 105);
    gfx_BlitRectangle(gfx_buffer, 0, 120, 160, 120);
    
    gfx_SetColor(lj_green);
    gfx_FillCircle(160, 120, 119);
    gfx_SetColor(lj_white);
    gfx_FillCircle(160, 120, 105);
    gfx_BlitRectangle(gfx_buffer, 160, 120, 160, 120);
    gfx_SetDrawScreen();
    gfx_PrintStringXY("By LogicalJoe", (LCD_WIDTH-gfx_GetStringWidth("By LogicalJoe")), (LCD_HEIGHT-9));
    gfx_PrintStringXY("Impossible Dial", 0, (LCD_HEIGHT-8));
    gfx_SetDrawBuffer();
}

void game(void) {
    /* game loop */
    srand(rtc_Time());
    way = 1;
    score = 0;
    color = 1 + rand() % 4;
    go = 1;
    a = 1;
    quit = false;
    prevkey = kb_Data[1] == kb_2nd;
    while (quit == false) {
        
        while (kb_AnyKey() && (quit == false)) {
            main_gamecode();
            gfx_Blit(gfx_buffer);
        }
        
        while ((!kb_AnyKey()) && (quit == false)) {
            main_gamecode();
            gfx_Blit(gfx_buffer);
            kb_Scan();
        }
        
        if (kb_Data[6] == kb_Clear)
            quit = true;
        /* switch directions */
        death();
        if (quit == true)
            score--;
        way = -way;
        score++;
        
        if (score >= highscore) {
            highscore = score;
            ti_CloseAll();
            file = ti_Open(Dial_AppVar, "w");
            if (file)
                ti_Write(&highscore, sizeof(highscore), 1, file);
        }
        if (line == true) {
            if (score > 50) {
                go = 50/5;
            }
            else {
                go = 1 + (score / 5);
            }
        }
        if (line == false) {
            if (score > 100) {
                go = 100/5;
            }
            else {
                go = 1 + (score / 5);
            }
        }
        
        
        colorprev = color;
        while (colorprev == color)
            color = 1 + rand() % 4;
        
        
    }
}

void death(void) {
    //red
    if (color == 1 && ((a > 270) || (a < 180)))
        quit = true;
    //blue
    if (color == 2 && (a < 270))
        quit = true;
    //yellow
    if (color == 3 && ((a > 180) || (a < 90)))
        quit = true;
    //green
    if (color == 4 && (a > 90))
        quit = true;
}

void passed(void) {
    if (way == -1) {
        if ((color == 1) && ((a < 180) && (c >= 180)))
            quit = true;
        if ((color == 2) && ((a < 270) && (c >= 270)))
            quit = true;
        if ((color == 3) && ((a < 90) && (c >= 90)))
            quit = true;
        if ((color == 4) && ((a < 360) && (c >= 0) && (a > 180) && (c < 180)))
            quit = true;
    }
    if (way == 1) {
        if ((color == 1) && ((a > 270) && (c <= 270)))
            quit = true;
        if ((color == 2) && ((a > 0) && (c <= 360) && (a < 180) && (c > 180)))
            quit = true;
        if ((color == 3) && ((a > 180) && (c <= 180)))
            quit = true;
        if ((color == 4) && ((a > 90) && (c <= 90)))
            quit = true;
    }
}

void high_Disp(void) {
    gfx_SetTextScale(1,1);
    gfx_SetTextXY(288,1);
    gfx_PrintInt(highscore, 4);
    gfx_SetTextScale(1,1);
}

void menu_draw(void) {
    quit = false;
    gfx_SetDrawScreen();
    gfx_SetColor(lj_white);
    gfx_FillCircle(160, 120, 105);
    srand(rtc_Time());
    for (a = 0; a < 150; a++) {
        color = 1 + rand() % 4;
        if (color == 1)
            gfx_SetColor(lj_red);
        if (color == 2)
            gfx_SetColor(lj_blue);
        if (color == 3)
            gfx_SetColor(lj_yellow);
        if (color == 4)
            gfx_SetColor(lj_green);
        gfx_FillCircle(160, 120, 50);
    }
    gfx_PrintStringXY("Any key - Play", (LCD_WIDTH-gfx_GetStringWidth("Anykey - Play")) / 2, 175);
    high_Disp();
    gfx_SetTextTransparentColor(255);
    gfx_SetTextScale(2,2);
    gfx_SetTextXY(130,113);
    gfx_PrintInt(score, 4);
    gfx_SetTextScale(1,1);
    gfx_SetTextTransparentColor(1);
    gfx_SetDrawBuffer();
}

void main_gamecode(void) {
    c = a;
    update();
    passed();
    render_dial();
    gfx_Blit(gfx_buffer);
    gfx_SetTextXY(1,1);
    gfx_PrintUInt(score, 4);
}
